<?xml version="1.0" encoding="UTF-8" ?> 
<xsl:stylesheet xmlns="http://www.ikg.uni-bonn.de/sig3d/pilot3d" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:str="http://example.com/namespace" xmlns:gml="http://www.opengis.net/gml" xml:inputSchema="CityGML">
    <xsl:output method="xml" version="1.0" indent="yes" encoding="UTF-8" /> 
    <xsl:template match="/*[1]">
        <FeatureCollection xsi:schemaLocation="http://www.ikg.uni-bonn.de/sig3d/pilot3d/UrbanGML.xsd">
            <xsl:for-each select="*:boundedBy">
                <xsl:call-template name="boundedBy"/>
            </xsl:for-each>
            <xsl:call-template name="AtomicTown"/>
        </FeatureCollection>
    </xsl:template>
    
    <!-- Templates der einzelnen Klassen -->
    
    <xsl:template name="_GraphicalMaterial">
        <xsl:for-each select="*:SimpleTexture">
            <xsl:call-template name="Texture"/>
        </xsl:for-each>
        <xsl:call-template name="X3DMaterial"/>
    </xsl:template>
    
    <xsl:template name="GenericMaterial">
    </xsl:template>
    
    <xsl:template name="Texture">
        <Texture>
            <xsl:for-each select="*:textureMap">
                <uri>
                    <xsl:value-of select="self::*:textureMap"/>
                </uri>    
            </xsl:for-each>
            <xsl:for-each select="*:repeat">
                <repeat>
                    <xsl:value-of select="self::*:repeat"/>
                </repeat>    
            </xsl:for-each>
            <xsl:for-each select="*:textureCoordinates">
                <textureCoordinates>
                    <xsl:value-of select="self::*:textureCoordinates"/>
                </textureCoordinates>    
            </xsl:for-each>
        </Texture>
    </xsl:template>
    
    <xsl:template name="X3DMaterial">
        <xsl:for-each select="*:Material">
            <xsl:if test="empty(ancestor::*:surfaceMember)">
            <X3DMaterial>
                <xsl:for-each select="*:ambientIntensity">
                    <ambientIntensity>
                        <xsl:value-of select="self::*:ambientIntensity"/>
                    </ambientIntensity>    
                </xsl:for-each>
                <xsl:for-each select="*:diffuseColor">
                    <diffuseColor>
                        <xsl:value-of select="normalize-space(self::*:diffuseColor)"/>
                    </diffuseColor>
                </xsl:for-each>
                <xsl:for-each select="*:emissiveColor">
                    <emissiveColor>
                        <xsl:value-of select="normalize-space(self::*:emissiveColor)"/>
                    </emissiveColor>
                </xsl:for-each>
                <xsl:for-each select="*:specularColor">
                    <specularColor>
                        <xsl:value-of select="normalize-space(self::*:specularColor)"/>
                    </specularColor>
                </xsl:for-each>
                <xsl:for-each select="*:transparency">
                    <transparency>
                        <xsl:value-of select="self::*:transparency"/>
                    </transparency>
                </xsl:for-each>
                <xsl:for-each select="*:shininess">
                    <shininess>
                        <xsl:value-of select="self::*:shininess"/>
                    </shininess>
                </xsl:for-each>
            </X3DMaterial>
            </xsl:if>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="Address">
        <xsl:for-each select="descendant::*:Address">
            <addressProperty>
                <Address>
                    <xsl:for-each select="*:deliveryPoint">
                        <deliveryPoint>
                            <xsl:value-of select="self::*:deliveryPoint"/>
                        </deliveryPoint>
                    </xsl:for-each>
                    <xsl:for-each select="*:postalCode">
                        <postalCode>
                            <xsl:value-of select="self::*:postalCode"/>
                        </postalCode>
                    </xsl:for-each>
                    <xsl:for-each select="*:city">
                        <city>
                            <xsl:value-of select="self::*:city"/>
                        </city>
                    </xsl:for-each>
                    <xsl:for-each select="*:country">
                        <country>
                            <xsl:value-of select="self::*:country"/>
                        </country>
                    </xsl:for-each>
                </Address>
            </addressProperty>
        </xsl:for-each>
        <xsl:call-template name="address"/>
    </xsl:template>
    
    <xsl:template name="address">
        <xsl:for-each select="descendant::*:address">
            <addressProperty>
                <Address>
                    <xsl:if test="exists(*:streetName)">
                        <deliveryPoint>
                            <xsl:value-of select="*:streetName"/>
                            <xsl:text> </xsl:text>
                            <xsl:value-of select="*:houseNumber"/>
                        </deliveryPoint>
                    </xsl:if>
                    <xsl:for-each select="*:zipCode">
                        <postalCode>
                            <xsl:value-of select="self::*:zipCode"/>
                        </postalCode>
                    </xsl:for-each>
                    <xsl:for-each select="*:city">
                        <city>
                            <xsl:value-of select="self::*:city"/>
                        </city>
                    </xsl:for-each>
                    <xsl:for-each select="*:country">
                        <country>
                            <xsl:value-of select="self::*:country"/>
                        </country>
                    </xsl:for-each>
                </Address>
            </addressProperty>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="ExternalReference">
        <xsl:for-each select="*:externalReference">
            <reference>
                <ExternalReference>
                    <informationSystem>
                        <xsl:value-of select="*:informationSystem"/>
                    </informationSystem>
                    <externalObjectReference>
                        <xsl:value-of select="*:externalObject/*:name"/>
                    </externalObjectReference>
                </ExternalReference>
            </reference>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="_Surface"/>
    
    <xsl:template name="MultiCurve">
        <xsl:for-each select="self::descendant">
            <xsl:copy/>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="_Solid"/>
    
    <xsl:template name="SurfaceLoD">
        <xsl:for-each select="*:lod1SurfaceProperty">
            <surfaceLoDProperty>
                <xsl:attribute name="levelOfDetail">1</xsl:attribute>
                <xsl:for-each select="*:Surface">
                    <xsl:call-template name="Surface"/>
                </xsl:for-each>
                <xsl:for-each select="*:CompositeSurface">
                    <xsl:call-template name="CompositeSurface"/>
                </xsl:for-each>
            </surfaceLoDProperty>
        </xsl:for-each>
        <xsl:for-each select="*:lod2SurfaceProperty">
            <surfaceLoDProperty>
                <xsl:attribute name="levelOfDetail">2</xsl:attribute>
                <xsl:for-each select="*:Surface">
                    <xsl:call-template name="Surface"/>
                </xsl:for-each>
                <xsl:for-each select="*:CompositeSurface">
                    <xsl:call-template name="CompositeSurface"/>
                </xsl:for-each>
            </surfaceLoDProperty>
        </xsl:for-each>
        <xsl:for-each select="*:lod3SurfaceProperty">
            <surfaceLoDProperty>
                <xsl:attribute name="levelOfDetail">3</xsl:attribute>
                <xsl:for-each select="*:Surface">
                    <xsl:call-template name="Surface"/>
                </xsl:for-each>
                <xsl:for-each select="*:CompositeSurface">
                    <xsl:call-template name="CompositeSurface"/>
                </xsl:for-each>
            </surfaceLoDProperty>
        </xsl:for-each>
        <xsl:for-each select="*:lod4SurfaceProperty">
            <surfaceLoDProperty>
                <xsl:attribute name="levelOfDetail">4</xsl:attribute>
                <xsl:for-each select="*:Surface">
                    <xsl:call-template name="Surface"/>
                </xsl:for-each>
                <xsl:for-each select="*:CompositeSurface">
                    <xsl:call-template name="CompositeSurface"/>
                </xsl:for-each>
                <xsl:for-each select="*:TexturedSurface">
                    <xsl:call-template name="TexturedSurface"/>
                </xsl:for-each>
            </surfaceLoDProperty>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="CurveLoD">
        <xsl:for-each select="*:lod1CurveProperty">
            <curveLoDProperty>
                <xsl:attribute name="levelOfDetail">1</xsl:attribute>
                <xsl:call-template name="MultiCurve"/>
            </curveLoDProperty>
        </xsl:for-each>
        <xsl:for-each select="*:lod2CurveProperty">
            <curveLoDProperty>
                <xsl:attribute name="levelOfDetail">2</xsl:attribute>
                <xsl:call-template name="MultiCurve"/>
            </curveLoDProperty>
        </xsl:for-each>
        <xsl:for-each select="*:lod3CurveProperty">
            <curveLoDProperty>
                <xsl:attribute name="levelOfDetail">3</xsl:attribute>
                <xsl:call-template name="MultiCurve"/>
            </curveLoDProperty>
        </xsl:for-each>
        <xsl:for-each select="*:lod4CurveProperty">
            <curveLoDProperty>
                <xsl:attribute name="levelOfDetail">4</xsl:attribute>
                <xsl:call-template name="MultiCurve"/>
            </curveLoDProperty>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="SolidLoD">
        <xsl:for-each select="*:lod1SolidProperty">
            <solidLoDProperty>
                <xsl:attribute name="levelOfDetail">1</xsl:attribute>
                <xsl:for-each select="*:Solid">
                    <xsl:call-template name="Solid"/>
                </xsl:for-each>
            </solidLoDProperty>
        </xsl:for-each>
        <xsl:for-each select="*:lod2SolidProperty">
            <solidLoDProperty>
                <xsl:attribute name="levelOfDetail">2</xsl:attribute>
                <xsl:for-each select="*:Solid">
                    <xsl:call-template name="Solid"/>
                </xsl:for-each>
            </solidLoDProperty>
        </xsl:for-each>
        <xsl:for-each select="*:lod3SolidProperty">
            <solidLoDProperty>
                <xsl:attribute name="levelOfDetail">3</xsl:attribute>
                <xsl:for-each select="*:Solid">
                    <xsl:call-template name="Solid"/>
                </xsl:for-each>
            </solidLoDProperty>
        </xsl:for-each>
        <xsl:for-each select="*:lod4SolidProperty">
            <solidLoDProperty>
                <xsl:attribute name="levelOfDetail">4</xsl:attribute>
                <xsl:for-each select="*:Solid">
                    <xsl:call-template name="Solid"/>
                </xsl:for-each>
            </solidLoDProperty>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="MaterialLoD">
            <xsl:if test="exists(ancestor::*:lod1SolidProperty)">
                <materialLoDProperty>
                    <xsl:attribute name="levelOfDetail">1</xsl:attribute>
                    <xsl:for-each select="*:appearanceProperty">
                        <xsl:call-template name="_GraphicalMaterial"/>
                    </xsl:for-each>  
                </materialLoDProperty>
            </xsl:if>
            <xsl:if test="exists(ancestor::*:lod2SolidProperty)">
                <materialLoDProperty>
                    <xsl:attribute name="levelOfDetail">2</xsl:attribute>
                    <xsl:for-each select="*:appearanceProperty">
                        <xsl:call-template name="_GraphicalMaterial"/>
                    </xsl:for-each>  
                </materialLoDProperty>
            </xsl:if>
            <xsl:if test="exists(ancestor::*:lod3SolidProperty)">
                <materialLoDProperty>
                    <xsl:attribute name="levelOfDetail">3</xsl:attribute>
                    <xsl:for-each select="*:appearanceProperty">
                        <xsl:call-template name="_GraphicalMaterial"/>
                    </xsl:for-each>  
                </materialLoDProperty>
            </xsl:if>
            <xsl:if test="exists(ancestor::*:lod4SolidProperty)">
                <materialLoDProperty>
                    <xsl:attribute name="levelOfDetail">4</xsl:attribute>
                    <xsl:for-each select="*:appearanceProperty">
                        <xsl:call-template name="_GraphicalMaterial"/>
                    </xsl:for-each>  
                </materialLoDProperty>
            </xsl:if>
            <xsl:if test="exists(ancestor::*:lod1SurfaceProperty)">
                <materialLoDProperty>
                <xsl:attribute name="levelOfDetail">1</xsl:attribute>
                    <xsl:for-each select="*:appearanceProperty">
                        <xsl:call-template name="_GraphicalMaterial"/>
                    </xsl:for-each>  
                </materialLoDProperty>
            </xsl:if>
            <xsl:if test="exists(ancestor::*:lod2SurfaceProperty)">
                <materialLoDProperty>
                <xsl:attribute name="levelOfDetail">2</xsl:attribute>
                    <xsl:for-each select="*:appearanceProperty">
                        <xsl:call-template name="_GraphicalMaterial"/>
                    </xsl:for-each>  
                </materialLoDProperty>
            </xsl:if>
        <xsl:if test="exists(ancestor::*:lod3SurfaceProperty)">
                <materialLoDProperty>
                <xsl:attribute name="levelOfDetail">3</xsl:attribute>
                    <xsl:for-each select="*:appearanceProperty">
                        <xsl:call-template name="_GraphicalMaterial"/>
                    </xsl:for-each>  
                </materialLoDProperty>
        </xsl:if>
        <xsl:if test="exists(ancestor::*:lod4SurfaceProperty)">
                <materialLoDProperty>
                <xsl:attribute name="levelOfDetail">4</xsl:attribute>
                    <xsl:for-each select="*:appearanceProperty">
                        <xsl:call-template name="_GraphicalMaterial"/>
                    </xsl:for-each>  
                </materialLoDProperty>
        </xsl:if>
        <xsl:if test="exists(ancestor::*:lod1CurveProperty)">
                <materialLoDProperty>
                <xsl:attribute name="levelOfDetail">1</xsl:attribute>
                    <xsl:for-each select="*:appearanceProperty">
                        <xsl:call-template name="_GraphicalMaterial"/>
                    </xsl:for-each>  
                </materialLoDProperty>
        </xsl:if>
        <xsl:if test="exists(ancestor::*:lod2CurveProperty)">
                <materialLoDProperty>
                <xsl:attribute name="levelOfDetail">2</xsl:attribute>
                    <xsl:for-each select="*:appearanceProperty">
                        <xsl:call-template name="_GraphicalMaterial"/>
                    </xsl:for-each>  
                </materialLoDProperty>
        </xsl:if>
        <xsl:if test="exists(ancestor::*:lod3CurveProperty)">
                <materialLoDProperty>
                <xsl:attribute name="levelOfDetail">3</xsl:attribute>
                    <xsl:for-each select="*:appearanceProperty">
                        <xsl:call-template name="_GraphicalMaterial"/>
                    </xsl:for-each>  
                </materialLoDProperty>
        </xsl:if>
        <xsl:if test="exists(ancestor::*:lod4CurveProperty)">
                <materialLoDProperty>
                <xsl:attribute name="levelOfDetail">4</xsl:attribute>
                    <xsl:for-each select="*:appearanceProperty">
                        <xsl:call-template name="_GraphicalMaterial"/>
                    </xsl:for-each>  
                </materialLoDProperty>
        </xsl:if>
    </xsl:template>
    
    <xsl:template name="Feature"/>
    
    <xsl:template name="FeatureCollection">
        <xsl:call-template name="Feature"/>
    </xsl:template>
    
    <xsl:template name="AbstractTown">
        <xsl:for-each select="name">
            <name>
                <xsl:value-of select="name"/>
            </name>
        </xsl:for-each>
        <xsl:for-each select="country">
            <country>
                <xsl:value-of select="country"/>
            </country>
        </xsl:for-each>
        <xsl:call-template name="FeatureCollection"/>
    </xsl:template>
    
    <xsl:template name="ComplexTown">
        <xsl:call-template name="AbstractTown"/>
    </xsl:template>
    
    <xsl:template name="UrbanFeature">
        <xsl:call-template name="Feature"/>
        <xsl:call-template name="ExternalReference"/>
    </xsl:template>
    
    <xsl:template name="AbstractTownParticle">
        <!-- <particleClass/>
        <particleDescription/> -->
        <xsl:call-template name="UrbanFeature"/>
    </xsl:template>
    
    <xsl:template name="AtomicTown">
        <featureMember>
            <AtomicTown>
                <xsl:call-template name="AbstractTown"/>
                <xsl:if test="exists(*:Building)">
                    <townParticleProperty>
                        <xsl:for-each select="*:Building">
                            <xsl:if test="exists(descendant::*:BuildingPart)">
                                <ComplexBuilding>
                                    <xsl:call-template name="ComplexBuilding"/>
                                </ComplexBuilding>
                            </xsl:if>
                            <xsl:if test="empty(descendant::*:BuildingPart)">
                                <xsl:call-template name="Building"/>
                            </xsl:if>
                        </xsl:for-each>
                    </townParticleProperty>
                    </xsl:if>
                    <xsl:for-each select="*:siteMember">
                    <xsl:for-each select="*:boundedBy">
                        <xsl:call-template name="boundedBy"/>
                    </xsl:for-each>
                        <townParticleProperty>
                            <xsl:for-each select="*:Building">
                                <xsl:if test="exists(descendant::*:BuildingPart)">
                                    <ComplexBuilding>
                                        <xsl:call-template name="ComplexBuilding"/>
                                    </ComplexBuilding>
                                </xsl:if>
                                <xsl:if test="empty(descendant::*:BuildingPart)">
                                    <xsl:call-template name="Building"/>
                                </xsl:if>
                            </xsl:for-each>
                        </townParticleProperty>    
                </xsl:for-each>                
            </AtomicTown>
        </featureMember>
    </xsl:template>
    
    <xsl:template name="Quarter">
        <townParticleProperty>
            <Quarter>
                <xsl:call-template name="AtomicTown"/>
            </Quarter>
        </townParticleProperty>
    </xsl:template>
    
    <xsl:template name="AbstractStructure">
        <xsl:call-template name="AbstractTownParticle"/>
        <xsl:call-template name="SurfaceLoD"/>
        <xsl:call-template name="CurveLoD"/>
        <xsl:call-template name="SolidLoD"/>
        <xsl:if test="exists(*:Material)">
            <x3dMaterialProperty>
                <xsl:call-template name="X3DMaterial"/>
            </x3dMaterialProperty>
        </xsl:if>
    </xsl:template>
    
    <xsl:template name="ComplexStructure">
        <xsl:call-template name="AbstractStructure"/>
    </xsl:template>
    
    <xsl:template name="AbstractStructureParticle"/>
    
    <xsl:template name="AtomicStructure">
        <xsl:call-template name="AbstractStructure"/>
        <xsl:call-template name="AbstractStructureParticle"/>
    </xsl:template>
    
    <xsl:template name="AbstractBuilding">
        <xsl:call-template name="AbstractStructure"/>
        <xsl:call-template name="Address"/>
    </xsl:template>
    
    <xsl:template name="ComplexBuilding">
        <xsl:call-template name="AbstractBuilding"/>
        <xsl:for-each select="*:consistsOfBuildingPart">
            <xsl:for-each select="*:BuildingPart">
                <buildingProperty>
                    <xsl:call-template name="Building"/>
                </buildingProperty>
            </xsl:for-each>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="AbstractBuildingParticle">
        <!-- <particleClass/>
        <particleDescription/> -->
        <xsl:if test="exists(*:Material)">
            <x3dMaterialProperty>
                <xsl:call-template name="X3DMaterial"/>
            </x3dMaterialProperty>
        </xsl:if>
        <xsl:call-template name="UrbanFeature"/>
        <xsl:for-each select="*:Building">
            <buildingParticleProperty>
                <xsl:call-template name="Building"/>
            </buildingParticleProperty>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="Building">
            <Building>
                <xsl:if test="exists(@*:id)">
                    <xsl:attribute name="gml:id">
                        <xsl:value-of select="@*:id"/>
                    </xsl:attribute>
                </xsl:if>
                <xsl:if test="exists(@*:storeysAboveground)">
                <storeys>
                    <xsl:value-of select="@*:storeysAboveground"/>
                </storeys>
                </xsl:if>
                <xsl:if test="exists(*:storeysAboveground)">
                    <storeys>
                        <xsl:value-of select="*:storeysAboveground"/>
                    </storeys>
                </xsl:if>
                <xsl:for-each select="*:address">
                    <xsl:call-template name="Address"/>
                </xsl:for-each>
                <xsl:call-template name="AbstractBuilding"/>
                <xsl:for-each select="*:boundedBy">
                    <xsl:call-template name="boundedBy"/>
                </xsl:for-each>
                <xsl:for-each select="*:boundingSurface">
                    <xsl:call-template name="boundedBy"/>
                </xsl:for-each>
                <xsl:for-each select="*:interiorRoomGroupProperty">
                    <xsl:call-template name="AtomicBuildingComponent"/>
                </xsl:for-each>
            </Building>
    </xsl:template>
    
    <xsl:template name="ResidentialHouse">
        <residents/>
        <tentments/>
        <xsl:call-template name="Building"/>
    </xsl:template>
    
    <xsl:template name="Garage">
        <parkingSlots/>
        <Building/>
    </xsl:template>
    
    <xsl:template name="AbstractBuildingComponent">
        <xsl:for-each select="*:measuredHeight">
            <componentHeight>
                <xsl:value-of select="self::*:measuredHeight"/>
            </componentHeight>
        </xsl:for-each>
        <xsl:call-template name="AbstractBuildingParticle"/>
        <xsl:call-template name="SurfaceLoD"/>
        <xsl:call-template name="SolidLoD"/>
    </xsl:template>
    
    <xsl:template name="ComplexBuildingComponent">
        <xsl:call-template name="AbstractBuildingComponent"/>
    </xsl:template>
    
    <xsl:template name="AbstractBuildingComponentParticle">
        <xsl:call-template name="UrbanFeature"/>
    </xsl:template>
    
    <xsl:template name="AbstractRoom">
        <xsl:call-template name="AbstractBuildingComponentParticle"/>
    </xsl:template>
    
    <xsl:template name="AtomicBuildingComponent">
        <xsl:call-template name="AbstractBuildingComponent"/>
        <xsl:call-template name="AbstractBuildingComponentParticle"/>
        <!-- Rooms, wenn fertig -->
    </xsl:template>
    
    <xsl:template name="Roof">
        <xsl:call-template name="AtomicBuildingComponent"/>
    </xsl:template>
    
    <xsl:template name="Storey">
        <buildingComponentParticleProperty>
            <Storey>
                <xsl:for-each select="*:name">
                    <storeyID>
                        <xsl:value-of select="self::*:name"/>
                    </storeyID>
                </xsl:for-each>
                <xsl:call-template name="AtomicBuildingComponent"/>
            </Storey>
        </buildingComponentParticleProperty>
    </xsl:template>
    
    <xsl:template name="Attic">
        <xsl:call-template name="Storey"/>
    </xsl:template>
    
    <xsl:template match="Basement">
        <xsl:call-template name="Storey"/>
    </xsl:template>
    
    <xsl:template name="AbstractBuildingModul">
        <xsl:call-template name="AbstractBuildingParticle"/>
        <xsl:call-template name="SurfaceLoD"/>
        <xsl:call-template name="CurveLoD"/>
        <xsl:call-template name="SolidLoD"/>
    </xsl:template>
    
    <xsl:template name="ComplexBuildingModul">
        <xsl:call-template name="AbstractBuildingModul"/>
    </xsl:template>
    
    <xsl:template name="Dormer">
        <!-- <roofType/> -->
        <xsl:call-template name="ComplexBuildingModul"/>
    </xsl:template>
    
    <xsl:template name="Balcony">
        <xsl:call-template name="ComplexBuildingModul"/>
    </xsl:template>
    
    <xsl:template name="AbstractBuildingModulParticle">
        <xsl:call-template name="UrbanFeature"/>
    </xsl:template>
    
    <xsl:template name="AtomicBuildingModul">
        <xsl:call-template name="AbstractBuildingModul"/>
        <xsl:call-template name="AbstractBuildingModulParticle"/>
    </xsl:template>
    
    <xsl:template name="Chimney">
        <xsl:call-template name="AtomicBuildingModul"/>
    </xsl:template>
    
    <xsl:template name="Opening">
        <xsl:call-template name="AtomicBuildingModul"/>
    </xsl:template>
    
    <xsl:template name="Window">
        <buildingParticleProperty>
        <Window>
            <xsl:if test="exists(@*:id)">
                <xsl:attribute name="gml:id">
                    <xsl:value-of select="@*:id"/>
                </xsl:attribute>
            </xsl:if>
            <!-- <casement/> -->
            <xsl:call-template name="Opening"/>
        </Window>
        </buildingParticleProperty>
    </xsl:template>
    
    <xsl:template name="Door">
        <buildingParticleProperty>
        <Door>
            <xsl:if test="exists(@*:id)">
                <xsl:attribute name="gml:id">
                    <xsl:value-of select="@*:id"/>
                </xsl:attribute>
            </xsl:if>
          <!--  <doorLeaf/>
            <movement/> -->
            <xsl:call-template name="Opening"/>
            <xsl:call-template name="Address"/>
        </Door>
        </buildingParticleProperty>
    </xsl:template>
    
    <xsl:template name="_Opening">
        <xsl:for-each select="*:Window">
            <xsl:call-template name="Window"/>
        </xsl:for-each>
        <xsl:for-each select="*:Door">
            <xsl:call-template name="Door"/>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="Stair">
        <xsl:call-template name="AtomicBuildingModul"/>
    </xsl:template>
    
    <xsl:template name="AbstractBuildingFace">
        <xsl:call-template name="AbstractBuildingParticle"/>
        <xsl:call-template name="SurfaceLoD"/>
        <xsl:call-template name="CurveLoD"/>
        <xsl:call-template name="MaterialLoD"/>
    </xsl:template>
    
    <xsl:template name="ComplexBuildingFace">
        <xsl:call-template name="AbstractBuildingFace"/>
    </xsl:template>
    
    <xsl:template name="AtomicBuildingFace">
        <xsl:call-template name="AbstractBuildingFace"/>
    </xsl:template>
    
    <xsl:template name="VisibleBuildingFace">
            <xsl:call-template name="AtomicBuildingFace"/>
        <xsl:for-each select="*:opening">
            <xsl:call-template name="_Opening"/>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="WallFace">
        <buildingParticleProperty>
        <WallFace>
            <xsl:if test="exists(@*:id)">
                <xsl:attribute name="gml:id">
                    <xsl:value-of select="@*:id"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:call-template name="VisibleBuildingFace"/>
        </WallFace>
        </buildingParticleProperty>
    </xsl:template>
    
    <xsl:template name="RoofFace">
    <buildingParticleProperty>
        <RoofFace>
            <xsl:if test="exists(@*:id)">
                <xsl:attribute name="gml:id">
                    <xsl:value-of select="@*:id"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:call-template name="VisibleBuildingFace"/>
        </RoofFace>
    </buildingParticleProperty>
    </xsl:template>
    
    <xsl:template name="InvisibleBuildingFace">
        <xsl:call-template name="AtomicBuildingFace"/>
    </xsl:template>
    
    <xsl:template name="CompletionFace">
        <buildingParticleProperty>
            <CompletionFace>
                <xsl:if test="exists(@*:id)">
                    <xsl:attribute name="gml:id">
                        <xsl:value-of select="@*:id"/>
                    </xsl:attribute>
                </xsl:if>
                <xsl:call-template name="InvisibleBuildingFace"/>
            </CompletionFace>
        </buildingParticleProperty>
    </xsl:template>
    
    <xsl:template name="FloorFace">
        <buildingParticleProperty>
        <FloorFace>
            <xsl:if test="exists(@*:id)">
                <xsl:attribute name="gml:id">
                    <xsl:value-of select="@*:id"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:call-template name="InvisibleBuildingFace"/>
        </FloorFace>
        </buildingParticleProperty>
    </xsl:template>
    
    <!-- Geometrie -->
    
    <xsl:template name="_BoundarySurface">
        <xsl:for-each select="*:RoofSurface">
            <xsl:call-template name="RoofFace"/>
        </xsl:for-each>
        <xsl:for-each select="*:CeilingSurface">
            <xsl:call-template name="RoofFace"/>
        </xsl:for-each>
        <xsl:for-each select="*:WallSurface">
            <xsl:call-template name="WallFace"/>
        </xsl:for-each>
        <xsl:for-each select="*:InnerWallSurface">
            <xsl:call-template name="WallFace"/>
        </xsl:for-each>
        <xsl:for-each select="*:FloorSurface">
            <xsl:call-template name="FloorFace"/>
        </xsl:for-each>
        <xsl:for-each select="*:InnerFloorSurface">
            <xsl:call-template name="FloorFace"/>
        </xsl:for-each>
        <xsl:for-each select="*:ClosingSurface">
            <xsl:call-template name="CompletionFace"/>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="boundedBy">
        <gml:boundedBy>
            <xsl:choose>
            <xsl:when test="exists(*:Envelope)">
                <xsl:for-each select="*:Envelope">
                    <gml:Envelope>
                        <xsl:if test="exists(@srsName)">
                            <xsl:attribute name="srsName">
                                <xsl:value-of select="@srsName"/>
                            </xsl:attribute>
                        </xsl:if>
                        <xsl:call-template name="convertCoordinates"/>
                    </gml:Envelope>
                </xsl:for-each>
            </xsl:when>
                <xsl:otherwise>
                    <xsl:call-template name="makeBoundingBox"/>
                </xsl:otherwise>
            </xsl:choose>
        </gml:boundedBy>
        <xsl:call-template name="_BoundarySurface"/>
    </xsl:template>
    
    <xsl:template name="convertCoordinates">
        <!-- convert all coordinates in gml:posList format -->
        <gml:posList>
            <xsl:attribute name="dimension">
                <xsl:for-each select="*:pos[1]">
                    <xsl:value-of select="count(index-of(string-to-codepoints(normalize-space(self::*:pos)),32))+1"/>
                </xsl:for-each>
                <xsl:for-each select="*:coord[1]">
                    <xsl:value-of select="3"/>
                </xsl:for-each>
                <xsl:for-each select="*:posList[1]">
                    <xsl:value-of select="@srsDimension"/>
                </xsl:for-each>
                <xsl:for-each select="*:coordinates[1]">
                    <xsl:value-of select="count(index-of(string-to-codepoints(translate(substring-before(normalize-space(self::*:coordinates), ' ') , ',' , ' ')),32))+1"/>
                </xsl:for-each>
                <xsl:for-each select="*:lowerCorner[1]">
                    <xsl:value-of select="3"/>
                </xsl:for-each>
            </xsl:attribute>
        <xsl:for-each select="*:coordinates">
            <!-- if gml:coordinates -->
            <xsl:choose>
                <xsl:when test="normalize-space(self::*:coordinates)">
                        <xsl:value-of select="concat(translate(substring-before(normalize-space(self::*:coordinates), ' ') , ',' , ' '),' ')"/>
                        <xsl:value-of select="concat(translate(substring-after(normalize-space(self::*:coordinates), ' ') , ',' , ' '),' ')"/>
                </xsl:when>
                <xsl:otherwise>
                        <xsl:value-of select="concat(translate(normalize-space(self::*:coordinates) , ',' ,' '),' ')"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
        <!-- end if gml:coordinates -->
        <xsl:for-each select="*:coord">
            <!-- if gml:coord -->
                <xsl:value-of select="concat(string-join((*:X,*:Y,*:Z),' '),' ')"/>
        </xsl:for-each>
        <!-- end if gml:coord -->
        <xsl:for-each select="*:pos">
            <!-- if gml:pos -->
                <xsl:value-of select="concat(self::*:pos,' ')"/>
        </xsl:for-each>
        <!-- end if gml:pos -->
            <xsl:for-each select="*:posList">
                <xsl:value-of select="self::*:posList"/>
            </xsl:for-each>
            <xsl:if test="exists(*:lowerCorner)">
                <xsl:value-of select="string-join((*:lowerCorner,*:upperCorner),' ')"/>
            </xsl:if>
        </gml:posList>
    </xsl:template>
    
    <xsl:template name="PolygonPatch">
        <gml:PolygonPatch>
            <xsl:for-each select="*:exterior">
                <gml:exterior>
                    <xsl:for-each select="*:LinearRing">
                        <xsl:call-template name="LinearRing"/>
                    </xsl:for-each>
                    <xsl:for-each select="*:Ring">
                        <xsl:copy/>
                    </xsl:for-each>
                </gml:exterior>
            </xsl:for-each>
            <xsl:for-each select="*:interior">
                <gml:interior>
                    <xsl:for-each select="*:LinearRing">
                        <xsl:call-template name="LinearRing"/>
                    </xsl:for-each>
                    <xsl:for-each select="*:Ring">
                        <xsl:copy/>
                    </xsl:for-each>
                </gml:interior>
            </xsl:for-each>
        </gml:PolygonPatch>
    </xsl:template>
    
    <xsl:template name="Polygon">
        <gml:Polygon>
            <xsl:for-each select="*:exterior">
                <gml:exterior>
                    <xsl:for-each select="*:LinearRing">
                        <xsl:call-template name="LinearRing"/>
                    </xsl:for-each>
                </gml:exterior>
            </xsl:for-each>
            <xsl:for-each select="*:interior">
                <gml:interior>
                    <xsl:for-each select="*:LinearRing">
                        <xsl:call-template name="LinearRing"/>
                    </xsl:for-each>
                    <xsl:for-each select="*:Ring">
                        <xsl:copy/>
                    </xsl:for-each>
                </gml:interior>
            </xsl:for-each>
        </gml:Polygon>
    </xsl:template>
    
    <xsl:template name="LinearRing">
        <gml:LinearRing>
            <xsl:call-template name="convertCoordinates"/>
            <xsl:for-each select="*:pointRep">
                <gml:pointRep>
                    <xsl:for-each select="*:Point">
                        <gml:Point>
                           <xsl:if test="exists(@*:id)">
                                <xsl:attribute name="gml:id">
                                    <xsl:value-of select="@*:id"/>
                                </xsl:attribute>
                           </xsl:if>
                            <xsl:call-template name="convertCoordinates"/>
                        </gml:Point>
                    </xsl:for-each>
                </gml:pointRep>
            </xsl:for-each>
        </gml:LinearRing>
    </xsl:template>
    
    <xsl:template name="Solid">
        <gml:Solid>
            <xsl:for-each select="*:MultiSolid">
                <xsl:copy/>
            </xsl:for-each>
            <xsl:if test="exists(@*:id)">
                <xsl:attribute name="gml:id">
                    <xsl:value-of select="@*:id"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:for-each select="*:exterior">
                <gml:exterior>
                    <xsl:for-each select="*:CompositeSurface">
                        <xsl:call-template name="CompositeSurface"/>
                    </xsl:for-each>
                    <xsl:for-each select="*:Surface">
                        <xsl:call-template name="Surface"/>
                    </xsl:for-each>
                </gml:exterior>
            </xsl:for-each>
            <xsl:for-each select="*:interior">
                <gml:interior>
                    <xsl:for-each select="*:CompositeSurface">
                        <xsl:call-template name="CompositeSurface"/>
                    </xsl:for-each>
                    <xsl:for-each select="*:Surface">
                        <xsl:call-template name="Surface"/>
                    </xsl:for-each>
                </gml:interior>
            </xsl:for-each>
        </gml:Solid>
    </xsl:template>
    
    <xsl:template name="Surface">
        <gml:Surface>
            <xsl:for-each select="*:MultiSurface">
                <xsl:copy/>
            </xsl:for-each>
          <xsl:if test="exists(@*:id)">
                <xsl:attribute name="gml:id">
                    <xsl:value-of select="@*:id"/>
                </xsl:attribute>
          </xsl:if>
            <xsl:for-each select="*:patches">
                <gml:patches>
                    <xsl:for-each select="*:PolygonPatch">
                        <xsl:call-template name="PolygonPatch"/>
                    </xsl:for-each>
                </gml:patches>
            </xsl:for-each>
        </gml:Surface>
    </xsl:template>
    
    <xsl:template name="TexturedSurface">
        <xsl:for-each select="*:baseSurface">
            <xsl:for-each select="*:Polygon">
                <xsl:call-template name="Polygon"/>
            </xsl:for-each>
        </xsl:for-each>
        <xsl:call-template name="MaterialLoD"/>
    </xsl:template>
    
    <xsl:template name="CompositeSolid">
        <gml:CompositeSolid>
            <xsl:if test="exists(@*:id)">
                <xsl:attribute name="gml:id">
                    <xsl:value-of select="@*:id"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:for-each select="*:solidMember">
                <gml:solidMember>
                    <xsl:for-each select="*:CompositeSolid">
                        <xsl:call-template name="CompositeSolid"/>
                    </xsl:for-each>
                    <xsl:for-each select="*:Solid">
                        <xsl:call-template name="Solid"/>
                    </xsl:for-each>
                </gml:solidMember>
            </xsl:for-each>
        </gml:CompositeSolid>
    </xsl:template>
    
    <xsl:template name="CompositeSurface">
        <gml:CompositeSurface>
            <xsl:if test="exists(@*:id)">
                <xsl:attribute name="gml:id">
                    <xsl:value-of select="@*:id"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="exists(@*:orientation)">
                <xsl:attribute name="orientation">
                    <xsl:value-of select="@*:orientation"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:for-each select="*:surfaceMember">
                <gml:surfaceMember>
                    <xsl:for-each select="*:CompositeSurface">
                        <xsl:call-template name="CompositeSurface"/>
                    </xsl:for-each>
                    <xsl:for-each select="*:OrientableSurface">
                        <xsl:call-template name="OrientableSurface"/>
                    </xsl:for-each>
                    <xsl:for-each select="*:Surface">
                        <xsl:call-template name="Surface"/>
                    </xsl:for-each>
                    <xsl:for-each select="*:Polygon">
                        <xsl:call-template name="Polygon"/>
                    </xsl:for-each>
                    <xsl:for-each select="*:TexturedSurface">
                        <xsl:call-template name="TexturedSurface"/>
                    </xsl:for-each>
                </gml:surfaceMember>
            </xsl:for-each>
         </gml:CompositeSurface>
    </xsl:template>
    
    <xsl:template name="OrientableSurface">
        <gml:OrientableSurface>
            <xsl:if test="exists(@*:id)">
                <xsl:attribute name="gml:id">
                    <xsl:value-of select="@*:id"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="exists(@*:orientation)">
                <xsl:attribute name="orientation">
                    <xsl:value-of select="@*:orientation"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:for-each select="*:surfaceMember">
                <gml:surfaceMember>
                    <xsl:for-each select="*:CompositeSurface">
                        <xsl:call-template name="CompositeSurface"/>
                    </xsl:for-each>
                    <xsl:for-each select="*:OrientableSurface">
                        <xsl:call-template name="OrientableSurface"/>
                    </xsl:for-each>
                    <xsl:for-each select="*:Surface">
                        <xsl:call-template name="Surface"/>
                    </xsl:for-each>
                    <xsl:for-each select="*:TexturedSurface">
                        <xsl:call-template name="TexturedSurface"/>
                    </xsl:for-each>
                </gml:surfaceMember>
            </xsl:for-each>
            <xsl:for-each select="*:baseSurface">
                <gml:baseSurface>
                <xsl:for-each select="*:Polygon">
                    <xsl:call-template name="Polygon"/>
                </xsl:for-each>
                </gml:baseSurface>
            </xsl:for-each>
        </gml:OrientableSurface>
    </xsl:template>
    
    <!-- zusaetzliche Funktionen -->
    
    <xsl:template name="makeBoundingBox">
        <gml:Envelope>
             <gml:posList>
                 <xsl:attribute name="dimension">3</xsl:attribute>
                 <xsl:if test="exists(descendant::*:posList)">
                     <xsl:value-of select="str:tokenmin(distinct-values(str:xvalues(string-join(descendant::*:posList,' '))))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmin(distinct-values(str:yvalues(string-join(descendant::*:posList,' '))))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmin(distinct-values(str:zvalues(string-join(descendant::*:posList,' '))))"/>
<!--                      <xsl:value-of select="str:listmax(string-join(distinct-values(str:xvalues(string-join(descendant::*:posList,' '))),' '))"/>
                      <xsl:value-of select="string(' ')"/>
                      <xsl:value-of select="str:listmax(string-join(distinct-values(str:yvalues(string-join(descendant::*:posList,' '))),' '))"/>
                      <xsl:value-of select="string(' ')"/>
                      <xsl:value-of select="str:listmax(string-join(distinct-values(str:zvalues(string-join(descendant::*:posList,' '))),' '))"/>-->
                     <xsl:value-of select="str:tokenmax(distinct-values(str:xvalues(string-join(descendant::*:posList,' '))))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmax(distinct-values(str:yvalues(string-join(descendant::*:posList,' '))))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmax(distinct-values(str:zvalues(string-join(descendant::*:posList,' '))))"/>
                 </xsl:if>
               <xsl:if test="exists(descendant::*:coordinates)">
                     <xsl:value-of select="str:tokenmin(distinct-values(str:xvalues(translate(string-join(descendant::*:coordinates,' '),',',' '))))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmin(distinct-values(str:yvalues(translate(string-join(descendant::*:coordinates,' '),',',' '))))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmin(distinct-values(str:zvalues(translate(string-join(descendant::*:coordinates,' '),',',' '))))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmax(distinct-values(str:xvalues(translate(string-join(descendant::*:coordinates,' '),',',' '))))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmax(distinct-values(str:yvalues(translate(string-join(descendant::*:coordinates,' '),',',' '))))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmax(distinct-values(str:zvalues(translate(string-join(descendant::*:coordinates,' '),',',' '))))"/>
                 </xsl:if>
                 <xsl:if test="exists(descendant::*:coord)">
                     <xsl:value-of select="str:tokenmin(string-join(descendant::*:X,' '))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmin(string-join(descendant::*:Y,' '))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmin(string-join(descendant::*:Z,' '))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmax(string-join(descendant::*:X,' '))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmax(string-join(descendant::*:Y,' '))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmax(string-join(descendant::*:Z,' '))"/>
                 </xsl:if>
                 <xsl:if test="exists(descendant::*:pos)">
                     <xsl:value-of select="str:tokenmin(distinct-values(str:xvalues(string-join(descendant::*:pos,' '))))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmin(distinct-values(str:yvalues(string-join(descendant::*:pos,' '))))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmin(distinct-values(str:zvalues(string-join(descendant::*:pos,' '))))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmax(distinct-values(str:xvalues(string-join(descendant::*:pos,' '))))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmax(distinct-values(str:yvalues(string-join(descendant::*:pos,' '))))"/>
                     <xsl:value-of select="string(' ')"/>
                     <xsl:value-of select="str:tokenmax(distinct-values(str:zvalues(string-join(descendant::*:pos,' '))))"/>
                 </xsl:if>
            </gml:posList>
        </gml:Envelope>        
    </xsl:template>

    <xsl:function name="str:xvalues">       
        <xsl:param name="list"/>
        <xsl:variable name="tokenlist" select="tokenize($list,'\s+')"/>
        <xsl:variable name="count" select="count($tokenlist)"/>
        <xsl:sequence select="for $i in 1 to ($count div 3) return subsequence($tokenlist, ((3*$i)-2),1)"/>
    </xsl:function>
    
    <xsl:function name="str:yvalues">
        <xsl:param name="list"/>
        <xsl:variable name="tokenlist" select="tokenize($list,'\s+')"/>
        <xsl:variable name="count" select="count($tokenlist)"/>
        <xsl:sequence select="for $i in 1 to ($count div 3) return subsequence($tokenlist, ((3*$i)-1),1)"/>
    </xsl:function>
    
    <xsl:function name="str:zvalues">
        <xsl:param name="list"/>
        <xsl:variable name="tokenlist" select="tokenize($list,'\s+')"/>
        <xsl:variable name="count" select="count($tokenlist)"/>
        <xsl:sequence select="for $i in 1 to ($count div 3) return subsequence($tokenlist, (3*$i),1)"/>
    </xsl:function>
    
    <xsl:function name="str:min">
        <xsl:param name="item1" as="xs:string"/>
        <xsl:param name="item2" as="xs:string"/>
        <xsl:sequence select="if ((number($item1)-number($item2))&gt;0) then $item2 else $item1"/>
    </xsl:function>
    
    <xsl:function name="str:listmin" as="xs:string">
        <xsl:param name="list" as="xs:string"/>
        <xsl:sequence select="if (contains($list,' ')) then str:min(substring-before($list,' '),str:listmin(substring-after($list,' '))) else $list"/>
    </xsl:function>
    
    <xsl:function name="str:max">
        <xsl:param name="item1" as="xs:string"/>
        <xsl:param name="item2" as="xs:string"/>
        <xsl:sequence select="if ((number($item1)-number($item2))&gt;0) then $item1 else $item2"/>
    </xsl:function>
    
    <xsl:function name="str:listmax" as="xs:string">
        <xsl:param name="list" as="xs:string"/>
        <xsl:sequence select="if (contains($list,' ')) then str:max(substring-before($list,' '),str:listmax(substring-after($list,' '))) else $list"/>
    </xsl:function>
    
    <xsl:function name="str:tokenmax">
        <xsl:param name="tokens"/>
        <xsl:sequence select="if(count($tokens)&gt;1) then str:max(string(subsequence($tokens,1,1)),string(str:tokenmax(remove($tokens,1)))) else $tokens"/>
    </xsl:function>
    
    <xsl:function name="str:tokenmin">
        <xsl:param name="tokens"/>
        <xsl:sequence select="if(count($tokens)&gt;1) then str:min(string(subsequence($tokens,1,1)),string(str:tokenmax(remove($tokens,1)))) else $tokens"/>
    </xsl:function>
        
</xsl:stylesheet>
