<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0">
    <xsl:output method="xml" indent="yes" version="1.0" encoding="UTF-8" /> 
    <xsl:template match="/*[1]">
        <ows>
            <xsl:attribute name="id">newWFS1.1.0</xsl:attribute>
            <xsl:apply-templates select="*:ServiceIdentification"/>
            <xsl:apply-templates select="*:OperationsMetadata"/>
            <xsl:apply-templates select="*:Filter_Capabilities"/>
        </ows>
    </xsl:template>
    
    <xsl:template match="*:ServiceIdentification">
        <service>
            <xsl:value-of select="*:ServiceType"/>
        </service>
        <version>
            <xsl:value-of select="*:ServiceTypeVersion"/>
        </version>
    </xsl:template>
    
    <xsl:template match="*:Filter_Capabilities">
        <filter>
            <xsl:if test="exists(//*:GeometryOperand)">
                <geometryOperands>
                    <xsl:for-each select="//*:GeometryOperand">
                        <geometryOperand>
                            <xsl:value-of select="self::*:GeometryOperand"/>
                        </geometryOperand>
                    </xsl:for-each>
                </geometryOperands>
            </xsl:if>
            <xsl:if test="exists(//*:SpatialOperator)">
                <spatialOperators>
                    <xsl:for-each select="//*:SpatialOperator">
                        <spatialOperator>
                            <xsl:value-of select="@*:name"/>
                        </spatialOperator>
                    </xsl:for-each>
                </spatialOperators>
            </xsl:if>
            <xsl:if test="exists(//*:ComparisonOperator)">
                <comparisonOperators>
                    <xsl:for-each select="//*:ComparisonOperator">
                        <comparisonOperator>
                            <xsl:value-of select="self::*:ComparisonOperator"/>
                        </comparisonOperator>
                    </xsl:for-each>
                </comparisonOperators>
            </xsl:if>
            <xsl:if test="exists(//*:SimpleArithmetic)">
                <arithmeticOperators>
                    <arithmeticOperator>Add</arithmeticOperator>
                    <arithmeticOperator>Sub</arithmeticOperator>
                    <arithmeticOperator>Mul</arithmeticOperator>
                    <arithmeticOperator>Div</arithmeticOperator>
                </arithmeticOperators>
            </xsl:if>
            <xsl:if test="exists(//*:LogicalOperators)">
                <logicalOperators>
                    <logicalOperator>And</logicalOperator>
                    <logicalOperator>Or</logicalOperator>
                    <logicalOperator>Not</logicalOperator>
                </logicalOperators>
            </xsl:if>
        </filter>
    </xsl:template>
    
    <xsl:template name="FeatureTypeListTemplate">
        <xsl:if test="exists(//*:FeatureTypeList)">
            <parameter id="TypeName" selected="true">
                <xsl:for-each select="//*:FeatureTypeList/*:FeatureType">
                    <value>
                        <xsl:value-of select="*:Name"/>
                    </value>
                </xsl:for-each>
            </parameter>
            <parameter id="namespace" selected="true">
                <xsl:for-each select="//*:FeatureTypeList/*:FeatureType">
                    <value>
                        <xsl:value-of select="substring-before(*:Name,':')"/>
                        <xsl:text>=</xsl:text>
                        <xsl:value-of select="namespace-uri-for-prefix(substring-before(*:Name,':'), *:Name)"/>
                    </value>
                </xsl:for-each>
            </parameter>
        </xsl:if>
    </xsl:template>
    
    <xsl:template name="GetFeatureTemplate">
        <parameter id="maxFeatures">
            <value>2</value>      
        </parameter>
        <parameter id="Filter" selected="false">
            <value>Generate Me</value>
        </parameter>
        <parameter id="FilterTransform" selected="false">
            <value>./xslt/webservice/filter/InteractiveInstruments.xslt</value>
        </parameter>
        <xsl:call-template name="FeatureTypeListTemplate"/>
    </xsl:template>
    
    <xsl:template name="DescribeFeatureTypeTemplate">
        <xsl:call-template name="FeatureTypeListTemplate"/>
    </xsl:template>
    
    <xsl:template name="TransactionTemplate">
<!--    <parameter id="transactionType" selected="true">
            <value>Insert</value> -->
            <!--      <value>Update</value>
      <value>Delete</value>
      <value>Native</value>-->
<!--    </parameter> -->
        <parameter id="file" selected="false">
            <value>FileToTransact.xml</value>
        </parameter>
<!--    <parameter id="Filter" selected="false">
            <value>Generate Me</value>
        </parameter> -->
    </xsl:template>
    
    <xsl:template match="*:OperationsMetadata">
        <xsl:for-each select="*:Operation">
            <request>
                <xsl:attribute name="id">
                    <xsl:value-of select="@*:name"/>
                </xsl:attribute>
                <xsl:for-each select="*:Parameter">
                    <parameter>
                        <xsl:attribute name="id">
                            <xsl:value-of select="@*:name"/>
                        </xsl:attribute>
                        <xsl:for-each select="*:Value">
                            <value>
                                <xsl:value-of select="self::*:Value"/>
                            </value>
                        </xsl:for-each>
                    </parameter>
                </xsl:for-each>
                <xsl:if test="compare(@*:name, 'GetFeature')=0">
                    <xsl:call-template name="GetFeatureTemplate"/>
                </xsl:if>
                <xsl:if test="compare(@*:name, 'DescribeFeatureType')=0">
                    <xsl:call-template name="DescribeFeatureTypeTemplate"/>
                </xsl:if>
                <xsl:if test="compare(@*:name, 'Transaction')=0">
                    <xsl:call-template name="TransactionTemplate"/>
                </xsl:if>
            </request>
        </xsl:for-each>
        <request id="SendSavedQuery" selected="true">
            <parameter id="file">
                <value>./queryTest.xml</value>
            </parameter>
        </request>
    </xsl:template>
    
</xsl:stylesheet>